﻿using System;
using System.Collections.Generic;
using System.Windows.Controls.Primitives;
using System.Runtime.InteropServices;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Interop;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using Microsoft.Samples.Office.UnifiedCommunications.Gadget.Events;

namespace Microsoft.Samples.Office.UnifiedCommunications.Gadget
{
    public partial class SettingsWindow : NavigationWindow
    {
        #region Variables

        private static SettingsWindow _instance = null;
        private static int _instanceCount = 0;

        #endregion

        public SettingsWindow()
        {
            _instance = this;
            
            _instanceCount++;

            InitializeCommands();
            InitializeComponent();

            this.MouseDown += new MouseButtonEventHandler(SettingsWindow_MouseDown);
            this.Unloaded += new RoutedEventHandler(SettingsWindow_Unloaded);
            this.Closing += new System.ComponentModel.CancelEventHandler(SettingsWindow_Closing);

            GadgetEventManager.Instance.ApplicationUnloadedEvent += new EventHandler(Instance_ApplicationUnloadedEvent);

            this.WindowStartupLocation = WindowStartupLocation.CenterScreen;
        }

        #region Properties

        public static int InstanceCount { get { return _instanceCount; } }

        public static SettingsWindow Instance
        {
            get
            {
                // if null, create and initialize the one and only window instance
                if (_instance == null)
                {
                    _instance = new SettingsWindow();
                    _instance.InitializeComponent();
                }
                return _instance;
            }
        }

        #endregion

        #region Loading

        private void NavigationWindow_Loaded(object sender, RoutedEventArgs e)
        {
            InitializeCommands();
        }

        #endregion

        #region Unloading

        void SettingsWindow_Unloaded(object sender, RoutedEventArgs e)
        {
            this.MouseDown -= new MouseButtonEventHandler(SettingsWindow_MouseDown);
            this.Closing -= new System.ComponentModel.CancelEventHandler(SettingsWindow_Closing);

            GadgetEventManager.Instance.ApplicationUnloadedEvent -= new EventHandler(Instance_ApplicationUnloadedEvent);
        }

        private void SettingsWindow_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            _instanceCount--;
        }

        void Instance_ApplicationUnloadedEvent(object sender, EventArgs e)
        {
            this.Close();
        }

        #endregion

        #region Other UI Functionality
        
        void SettingsWindow_MouseDown(object sender, MouseButtonEventArgs e)
        {
            this.DragMove();
        }

         private void InitializeCommands()
        {
            CommandBindings.Add(new CommandBinding(CloseWindow, CloseWindowExecuted));
            CommandBindings.Add(new CommandBinding(ToggleMinimizedState, MinimizeWindowExecuted));
            CommandBindings.Add(new CommandBinding(ToggleMaximizedState, MaximizeWindowExecuted));
        }

        public static RoutedUICommand CloseWindow = new RoutedUICommand("Close", "CloseWindow", typeof(SettingsWindow));

        private void CloseWindowExecuted(object sender, ExecutedRoutedEventArgs e)
        {
            Close();
        }

        public static RoutedUICommand ToggleMaximizedState = new RoutedUICommand("Maximize", "ToggleMaximizedState", typeof(SettingsWindow));
        private void MaximizeWindowExecuted(object sender, ExecutedRoutedEventArgs e)
        {
            SettingsWindow.Instance.WindowState = 
                (SettingsWindow.Instance.WindowState == WindowState.Maximized) ? WindowState.Normal : WindowState.Maximized;
        }

        public static RoutedUICommand ToggleMinimizedState = new RoutedUICommand("Minimize", "ToggleMinimizedState", typeof(SettingsWindow));
        private void MinimizeWindowExecuted(object sender, ExecutedRoutedEventArgs e)
        {
            SettingsWindow.Instance.WindowState = 
                (SettingsWindow.Instance.WindowState == WindowState.Minimized) ? WindowState.Normal : WindowState.Minimized;
        }       

        #endregion
    }
}